/*	XOP1.c -- A sample Igor external operation

	XOP1 is a very simple Igor XOP (external operation) designed to show you how to get
	started writing XOPs.
	
	The syntax for XOP1 is:
		XOP1 wave
	
	It adds 1 to the specified wave.
	It generates an error message if the wave is complex or text (not numeric).
*/

#include "XOPStandardHeaders.h"			// Include ANSI headers, Mac headers, IgorXOP.h, XOP.h and XOPSupport.h

#include "XOP1.h"

// Global Variables (none)

#include "Python.h"

static int PythonFunc(char *fileName, char *funcName)//, char *args[])
{
    PyObject *pName, *pModule, *pDict, *pFunc;
    PyObject *pArgs, *pValue;
    int i;
	char temp[256];
	
	Py_Initialize();
    pName = PyString_FromString(fileName);
    /* Error checking of pName left out */

    pModule = PyImport_Import(pName);
    Py_DECREF(pName);

    if (pModule != NULL) {
        pFunc = PyObject_GetAttrString(pModule, funcName);
        /* pFunc is a new reference */

        if (pFunc && PyCallable_Check(pFunc)) {
            /*pArgs = PyTuple_New(sizeof(args));
            for (i = 0; i < sizeof(args); ++i) {
                pValue = PyInt_FromLong(atoi(args[i]));
                if (!pValue) {
                    Py_DECREF(pArgs);
                    Py_DECREF(pModule);
                    fprintf(stderr, "Cannot convert argument\n");
                    return 1;
                }
                // pValue reference stolen here: 
                PyTuple_SetItem(pArgs, i, pValue);
            }
            */
			pArgs = PyTuple_New(0);
			pValue = PyObject_CallObject(pFunc, pArgs);
            Py_DECREF(pArgs);
            if (pValue != NULL) {
                sprintf(temp,"Result of call: %ld\r", PyInt_AsLong(pValue));
                XOPNotice(temp); 
				Py_DECREF(pValue);
            }
            else {
                Py_DECREF(pFunc);
                Py_DECREF(pModule);
                PyErr_Print();
                sprintf(temp,"Call failed\r");
				XOPNotice(temp);
				return 1;
            }
        }
        else {
            if (PyErr_Occurred())
                PyErr_Print();
            sprintf(temp, "Cannot find function \"%s\"\r", funcName);
			XOPNotice(temp);
		}
        Py_XDECREF(pFunc);
        Py_DECREF(pModule);
    }
    else {
        PyErr_Print();
        sprintf(temp, "Failed to load \"%s\"\r", fileName);
        XOPNotice(temp);
		return 1;
    }
    Py_Finalize();
    return 0;
}

// Operation template: PythonFunc string:fileName, string:funcName

// Runtime param structure for PythonFunc operation.
#pragma pack(2)	// All structures passed to Igor are two-byte aligned.
struct PythonFuncRuntimeParams {
	// Flag parameters.

	// Main parameters.

	// Parameters for simple main group #0.
	int fileNameEncountered;
	Handle fileName;
	int fileNameParamsSet[1];

	// Parameters for simple main group #1.
	int funcNameEncountered;
	Handle funcName;
	int funcNameParamsSet[1];

	// These are postamble fields that Igor sets.
	int calledFromFunction;					// 1 if called from a user function, 0 otherwise.
	int calledFromMacro;					// 1 if called from a macro, 0 otherwise.
};
typedef struct PythonFuncRuntimeParams PythonFuncRuntimeParams;
typedef struct PythonFuncRuntimeParams* PythonFuncRuntimeParamsPtr;
#pragma pack()	// All structures passed to Igor are two-byte aligned.

static int
ExecutePythonFunc(PythonFuncRuntimeParamsPtr p)
{
	int err = 0;
	char fileName_[50];
	char funcName_[50];
	

	// Main parameters.

	if (p->fileNameEncountered) {
		// Parameter: p->fileName (test for NULL handle before using)
	}

	if (p->funcNameEncountered) {
		// Parameter: p->funcName (test for NULL handle before using)
	}
	
	GetCStringFromHandle(p->fileName, fileName_,sizeof(fileName_));
	GetCStringFromHandle(p->funcName, funcName_,sizeof(funcName_));
	err=PythonFunc(fileName_,funcName_);
	return err;
}

static int
RegisterPythonFunc(void)
{
	const char* cmdTemplate;
	const char* runtimeNumVarList;
	const char* runtimeStrVarList;

	// NOTE: If you change this template, you must change the PythonFuncRuntimeParams structure as well.
	cmdTemplate = "PythonFunc string:fileName, string:funcName";
	runtimeNumVarList = "";
	runtimeStrVarList = "";
	return RegisterOperation(cmdTemplate, runtimeNumVarList, runtimeStrVarList, sizeof(PythonFuncRuntimeParams), (void*)ExecutePythonFunc, 0);
}

static int
RegisterOperations(void)		// Register any operations with Igor.
{
	int result;
	
	// Register XOP1 operation.

	// Register CallPython operation.
	if (result = RegisterPythonFunc())
		return result;
	
	// There are no more operations added by this XOP.
		
	return 0;
}

/*	XOPEntry()

	This is the entry point from the host application to the XOP for all
	messages after the INIT message.
*/
static void
XOPEntry(void)
{	
	long result = 0;
	
	switch (GetXOPMessage()) {
		// We don't need to handle any messages for this XOP.
	}
	SetXOPResult(result);
}

/*	main(ioRecHandle)

	This is the initial entry point at which the host application calls XOP.
	The message sent by the host must be INIT.
	
	main does any necessary initialization and then sets the XOPEntry field of the
	ioRecHandle to the address to be called for future messages.
*/
HOST_IMPORT void
main(IORecHandle ioRecHandle)
{
	int result;
	
	XOPInit(ioRecHandle);							// Do standard XOP initialization.

	SetXOPEntry(XOPEntry);							// Set entry point for future calls.

	if (result = RegisterOperations()) {
		SetXOPResult(result);
		return;
	}
	
	SetXOPResult(0);
}
